import "ExoPlugins.AssistantParse";

function LoadPlugin()
    Player = Turbine.Gameplay.LocalPlayer.GetInstance();
    Callbacks = {};
    SkillList = Player:GetTrainedSkills();
    SkillsTable = {};
    PlayerEffects = Player:GetEffects();
    Settings = {};
    Updater = Updater();
    Font = ConfigureFontTwo();
    CombatStartTime = 0;
    SkillCount = 0;
    if Turbine.PluginData.Load(Turbine.DataScope.Character, "AssistantParseSettings") == nil then
        Settings = {
            ["General"] = {
                ["Position"] = {
                    [1] = 0;
                    [2] = 0;
                }
            }
        }
    else
        Settings = LoadData(Turbine.DataScope.Character, "AssistantParseSettings");
    end
    local screenWidth, screenHeight = Turbine.UI.Display:GetSize();
    SkillWindow = SkillWindow(Settings["General"]["Position"][1]*screenWidth, Settings["General"]["Position"][2]*screenHeight);
	for i = 1, SkillList:GetCount(), 1 do
		local item = SkillList:GetItem(i);
		local name = item:GetSkillInfo():GetName();
		local ID = item:GetSkillInfo():GetIconImageID();

		Callbacks[name] = {}

        SkillsTable[name] = item
		table.insert(Callbacks[name], AddCallback(item, "ResetTimeChanged", function(sender, args)
            if (-Round(Turbine.Engine.GetGameTime()-item:GetResetTime(), 1) == item:GetCooldown()) then
                SkillWindow:AddSkill(ID);
                SkillCount = SkillCount + 1;
            end
		end));
	end

    --Workaround for Wardens to display gambit default and Rapid Techniques mastery usage
    if GetClass(Player) == "Warden" then

        MasteryStrings = {
            ["Spear and Spear"] = 1091659088,
            ["Spear and Shield"] = 1091471246,
            ["Spear and Fist"] = 1091471238,
            ["Shield and Spear"] = 1091471250,
            ["Shield and Shield"] = 1091659090,
            ["Shield and Fist"] = 1091471231,
            ["Fist and Spear"] = 1091471240,
            ["Fist and Shield"] = 1091471243,
            ["Fist and Fist"] = 1091659089,
        };

        ChatHandler = Turbine.Chat;
        ChatListener = function(f, args)
            if args ~= nil then
                if args.ChatType == Turbine.ChatType.PlayerCombat then
                    local source,skillName,target = string.match(string.gsub(string.gsub(args["Message"],"<rgb=#......>(.*)</rgb>","%1"),"^%s*(.-)%s*$", "%1"),"^(.*) applied a benefit with (.*) on (.*)")
                    if skillName ~= nil then
                        if MasteryStrings[skillName] ~= nil then
                            SkillWindow:AddSkill(MasteryStrings[skillName]);
                            SkillCount = SkillCount+1;
                        end
                    end
                end
            end
        end

        EffectAddedCallback = AddCallback(PlayerEffects, "EffectAdded", function(sender, args)
            local effect = PlayerEffects:Get(args.Index);
            local effectName = effect:GetName();
            if effectName == "Rapid Techniques" then
                AddCallback(ChatHandler, "Received", ChatListener);
            end
        end);

        EffectRemovedCallback = AddCallback(PlayerEffects, "EffectRemoved", function(sender, args)
            local effect = args.Effect;
            if effect ~= nil then 
                local effectName = effect:GetName();
                if effectName == "Rapid Techniques" then
                    RemoveCallback(ChatHandler, "Received", ChatListener);
                end
            end
        end);

        for i = 1, SkillList:GetCount(), 1 do
            local item = SkillList:GetItem(i);
            local name = item:GetSkillInfo():GetName();
            if (name == "Gambit Default") then
                local ID = item:GetSkillInfo():GetIconImageID();
                table.insert(Callbacks[name], AddCallback(item, "ResetTimeChanged", function(sender, args)
                        SkillWindow:AddSkill(ID);
                        SkillCount = SkillCount + 1;
                end));
            end
        end
    end

    --Workaround for Hunters. Since QS has no CD we need to track combat log.
    if GetClass(Player) == "Hunter" then
        ChatHandler = Turbine.Chat;
        ChatListener = function(f, args)
            if args ~= nil then
                if args.ChatType == Turbine.ChatType.PlayerCombat then
                    local source,_,skillName,target,_ = string.match(string.gsub(string.gsub(args["Message"],"<rgb=#......>(.*)</rgb>","%1"),"^%s*(.-)%s*$", "%1"),"^(.*) scored a (.*)hit with (.*) on (.*) for (.-) (.*)%.$")
                    if skillName ~= nil then
                        if string.find(skillName, "Quick Shot") then
                            SkillWindow:AddSkill(1090553925);
                            SkillCount = SkillCount + 1;
                        end
                    end
                end
            end
        end
        AddCallback(ChatHandler, "Received", ChatListener);
    end

    --Just checking if we're currently in combat when loading plugin, mostly for my own test stuff
    if Player:IsInCombat() == true then
        SkillCount = 0;
        CombatStartTime = Turbine.Engine.GetGameTime();
        AddCallback(Updater, "Tick", SkillWindow.UpdateAPM);
    end

    CombatHandler = function(f, args)
        if Player:IsInCombat() == true then
            SkillCount = 0;
            CombatStartTime = Turbine.Engine.GetGameTime();
            AddCallback(Updater, "Tick", SkillWindow.UpdateAPM);
        else
            RemoveCallback(Updater, "Tick", SkillWindow.UpdateAPM);
        end
    end
    AddCallback(Player, "InCombatChanged", CombatHandler)


end

LoadPlugin();

function UnloadPlugin()
    SaveData(Turbine.DataScope.Character, "AssistantParseSettings", Settings);
    for key, value in pairs(Callbacks) do
        for _, callback in pairs(value) do
            RemoveCallback(SkillsTable[key], "ResetTimeChanged", callback);
        end
    end
    if GetClass(Player) == "Warden" then
        RemoveCallback(PlayerEffects, "EffectAdded", EffectAddedCallback);
        RemoveCallback(PlayerEffects, "EffectRemoved", EffectRemovedCallback);
        RemoveCallback(ChatHandler, "Received", ChatListener);
    end
    if GetClass(Player) == "Hunter" then
        RemoveCallback(ChatHandler, "Received", ChatListener);
    end
end

plugin.Load=function(sender, args)
	Turbine.Shell.WriteLine("<rgb=#FF5555><" .. plugin:GetName() .. "></rgb> V." .. plugin:GetVersion() .. " loaded.");
end

plugin.Unload=function()
    UnloadPlugin();
	Turbine.Shell.WriteLine("<rgb=#FF5555><AssistantParse></rgb> Unload Complete");
end